#include "kernelsolver.h"

const double alpha=1.0;
const double beta=0.0;

float solve_cublas(double *mA, double *V, double *Result, int n){
  double *dev_mA;
  double *dev_V;
  double *dev_Result;

  /*выделение памяти*/
  cudaMalloc((void**) &dev_mA, n * n * sizeof(*mA));
  cudaMalloc((void**) &dev_V, n * sizeof(*V));
  cudaMalloc((void**) &dev_Result, n * sizeof(*Result));

  cublasHandle_t handle;
  cublasCreate(&handle);

  /*копирование в память gpu*/
  cublasSetMatrix(n, n, sizeof(*mA), mA, n, dev_mA, n);
  cublasSetVector(n, sizeof(*V), V, 1, dev_V, 1);

  cudaEvent_t start,stop;
  float gputime=0;

  cudaEventCreate(&start);
  cudaEventCreate(&stop);

  cudaEventRecord(start,0);

  /*произведение матрицы на вектор*/
  cublasDgemv(handle, CUBLAS_OP_T, n, n, &alpha, dev_mA, n, dev_V, 1, &beta, dev_Result, 1);

  cudaEventRecord(stop,0);
  cudaEventSynchronize(stop);
  cudaEventElapsedTime(&gputime,start,stop);

  /*копирование в память cpu*/
  cublasGetVector(n, sizeof(*Result), dev_Result, 1, Result, 1);

  /*очистка памяти*/
  cublasDestroy(handle);
  cudaFree(dev_mA);
  cudaFree(dev_V);
  cudaFree(dev_Result);
  cudaDeviceReset();

  return gputime;
}
